#include "image.h"

Image::Image()
{
	printf("allocating image\n");
    data = NULL;
	rgb_data = NULL;
    width = 0;
    height = 0;
}

double Image::Get(int x, int y) 
{
	if (x >= 0 && x < width && y >= 0 && y < height)
		return data[x + y * width];
	else 
	{
		if (x < 0) 
			x = 0;                
		else if (x >= width) 
			x = width-1;
			
		if (y < 0) 
			y = 0; 
		else if (y >= height) 
			y = height-1; 
			
		return data[x + y * width]; 
	}
}

void Image::Set(int x, int y, double value) 
{
	if (x >= 0 && x < width && y >= 0 && y < height)
		data[x + y*width] = value;
}

void Image::Acc(int x, int y, double value) 
{
	if (x >= 0 && x < width && y >= 0 && y < height) 
		data[x + y*width] += value;
}
	
void Image::Normalize(double minval, double maxval) 
{
	int i, j;
    double max;
    double min;
    double newrange;
    double oldrange; 
    max = 0;
    min = INT_MAX;
	
    for (i = 0; i < width; i++) 
	{
        for (j = 0; j < height; j++) 
		{
            if (Get(i,j) > max)
                max = Get(i,j);
            if (Get(i,j) < min)
                min = Get(i,j); 
        }
    }
    
    newrange = maxval - minval;
    oldrange = max - min; 
    
    for (i = 0; i < width; i++) 
	{
        for (j = 0; j < height; j++) 
		{
            double old = Get(i,j);
            double n;
            n = (((old - min) / oldrange) * newrange) + minval;
            Set(i,j,n); 
        }
    }

    printf("Minval was %g\n", min); 
    printf("Maxval was %g\n", max); 
}

void Image::WriteCMUfile(char *filename) 
{
	int i, j, v;
	unsigned char character;
    FILE *output = fopen(filename, "w");
	
	character = 'A';
	fwrite(&character, 1, 1, output);
	character = 'C';
	fwrite(&character, 1, 1, output);
	character = 'K';
	fwrite(&character, 1, 1, output);
	character = '\r';
	fwrite(&character, 1, 1, output);
	character = 1;
	fwrite(&character, 1, 1, output);
	character = (unsigned char)width;
	fwrite(&character, 1, 1, output);
	character = (unsigned char)height;
	fwrite(&character, 1, 1, output);
	printf("write %d x %d\n", width, height);
	
    for (j = 0; j < height; j++) 
	{
		character = 2;
		fwrite(&character, 1, 1, output);
        for (i = 0; i < width; i++) 
		{
            v = (int)floor(Get(i,j) + 0.5); 
            fwrite(&v, 1, 1, output); // red
            fwrite(&v, 1, 1, output); // green
            fwrite(&v, 1, 1, output); // blue
        }
    }
	
    fclose(output); 
}

void Image::ReadCMUfile(char *filename)
{
	int i, j;
	unsigned char temp;
	unsigned char pixel1, pixel2, pixel3;
    FILE *input = fopen(filename, "rb");
	
	fread(&temp,   1, 1, input); // A
	fread(&temp,   1, 1, input); // C
	fread(&temp,   1, 1, input); // K
	fread(&temp,   1, 1, input); // \r
	fread(&temp,   1, 1, input); // 01 for new frame
	fread(&width,  1, 1, input); // 88
	fread(&height, 1, 1, input); // 143

    Allocate(width, height);
    printf("Read image %d by %d\n", width, height); 
    
    for (j = 0; j < height; j++) 
	{
		fread(&temp, 1, 1, input); 			// 02 for new row
        for (i = 0; i < width; i++)
		{
			fread(&pixel1, 1, 1, input); // red
			fread(&pixel2, 1, 1, input); // green
			fread(&pixel3, 1, 1, input); // blue
			this->Set(i,j,(double)pixel1); 
			rgb_data[(width*(height-j-1)+(i))*3+0] = pixel1;
			rgb_data[(width*(height-j-1)+(i))*3+1] = pixel2;
			rgb_data[(width*(height-j-1)+(i))*3+2] = pixel3;
        }
    }
	
    fclose(input);
}

void Image::Allocate(int width, int height) 
{
	if (data != NULL)
	{
		free(data);
		data = NULL;
		free(rgb_data);
		rgb_data = NULL;
		printf("free image data\n");
	}
    data = (double*)malloc(width * height * sizeof(double));
	rgb_data = (unsigned char *)malloc(width * 2 * height * 3 * sizeof(unsigned char));
    memset(data, 0, width * height * sizeof(double));
    this->width = width;
    this->height = height; 
}

void Image::Multacc(Image *kernel, int x, int y, double mult) 
{
    int i, j, sx, sy;
    sx = x - kernel->width/2;
    sy = y - kernel->height/2;

    for (i = sx; i < sx+kernel->width; i++) 
	{
        for (j = sy; j < sy+kernel->height; j++) 
		{
            Acc(i,j,kernel->Get(i-sx, j-sy)*mult); 
        }
    }
}

void Image::Convolve(Image *kernel, Image *input, int upsample)
{
	int i, j, v, sx, sy;
    sx = (int)floor((kernel->width+1)/2);
    sy = (int)floor((kernel->height+1)/2);

	for (i = -sx; i < input->width+kernel->width/2; i++) 
	{
		for (j = -sy; j < input->height+kernel->height/2; j++) 
		{
			Multacc(kernel, (i+1)*upsample, (j+1)*upsample, input->Get(i,j));
		}
	}
	
	// update rgb_data
	for (j = 0; j < height; j++) 
	{
		for (i = 0; i < width; i++)
		{
            v = (int)floor(Get(i,j) + 0.5); 
            if (v < 0) v = 0;
            if (v > 255) v = 255;

			rgb_data[(width*(height-j-1)+(i))*3+0] = v;
			rgb_data[(width*(height-j-1)+(i))*3+1] = v;
			rgb_data[(width*(height-j-1)+(i))*3+2] = v;
		}
	}
}

void Image::Convolve_div(Image *kernel, Image *input, int div)
{
	int i, j, v, sx, sy;
    sx = (int)floor((kernel->width+1)/2);
    sy = (int)floor((kernel->height+1)/2);

	for (i = -sx; i < input->width+kernel->width/2; i++) {
		for (j= -sy; j < input->height+kernel->height/2; j++) {
			Multacc(kernel, (i+1), (j+1), input->Get(i,j)/div);
		}
	}

	// update rgb_data
	for (j = 0; j < height; j++) 
	{
		for (i = 0; i < width; i++)
		{
            v = (int)floor(Get(i,j) + 0.5); 
            if (v < 0) v = 0;
            if (v > 255) v = 255;
			rgb_data[(width*(height-j-1)+(i))*3+0] = v;
			rgb_data[(width*(height-j-1)+(i))*3+1] = v;
			rgb_data[(width*(height-j-1)+(i))*3+2] = v;
		}
	}
}

