#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <limits.h> 

#include "image_kernels.h"

void HSobel::Create(void) {
    this->Allocate(3,3);
	this->Set(0, 0, 1);
	this->Set(1, 0, 0);
	this->Set(2, 0,-1);
	this->Set(0, 1, 2);
	this->Set(1, 1, 0);
	this->Set(2, 1,-2);
	this->Set(0, 2, 1);
	this->Set(1, 2, 0);
	this->Set(2, 2,-1);
}

void VSobel::Create(void) {
    this->Allocate(3,3);
	this->Set(0, 0, 1);
	this->Set(1, 0, 2);
	this->Set(2, 0, 1);
	this->Set(0, 1, 0);
	this->Set(1, 1, 0);
	this->Set(2, 1, 0);
	this->Set(0, 2,-1);
	this->Set(1, 2,-2);
	this->Set(2, 2,-1);
}

void Blur::Create(void) 
{
    this->Allocate(3,3);
	this->Set(0, 0, 1);
	this->Set(1, 0, 2);
	this->Set(2, 0, 1);
	this->Set(0, 1, 2);
	this->Set(1, 1, 4);
	this->Set(2, 1, 2);
	this->Set(0, 2, 1);
	this->Set(1, 2, 2);
	this->Set(2, 2, 1);
}

void Sharpen::Create(void) {
    this->Allocate(3,3);
	this->Set(0, 0, -1);
	this->Set(1, 0, -1);
	this->Set(2, 0, -1);
	this->Set(0, 1, -1);
	this->Set(1, 1,  9);
	this->Set(2, 1, -1);
	this->Set(0, 2, -1);
	this->Set(1, 2, -1);
	this->Set(2, 2, -1);
}

void Sinc::Create(double maxrange, double sample) 
{
#define OVERSAMPLE 25.0
	double i, j, k, l, x, y, sum;
    double allsum; 
    this->Allocate(
        (int)ceil(maxrange*sample),
        (int)ceil(maxrange*sample));

    allsum = 0; 
    printf("Sinc size %d by %d\n", this->width, this->height); 
    for (i = 0; i < maxrange * sample; i++) 
	{
        for (j = 0; j < maxrange * sample; j++) 
		{
            x = (i / sample) - maxrange/2.0;
            y = (j / sample) - maxrange/2.0;

            sum = 0; 
            for (k = x; k < x+1.0; k += 1.0/OVERSAMPLE) 
			{
                for (l = y ; l < y+1.0; l += 1.0/OVERSAMPLE) 
				{
                    sum += sinc(k) * sinc(l); 
                }
            }
            sum /= OVERSAMPLE*OVERSAMPLE; 

            allsum += sum; 
            this->Set((int)i,(int)j,sum);
        }
    }
    printf("Allsum is %g\n", allsum); 
}
