//#pragma comment( linker, "/subsystem:\"windows\" /entry:\"mainCRTStartup\"" ) 
#include <windows.h>
#include "glut.h"
#include <stdio.h>
#include "serial.h"
#include "image.h"
#include "image_processing.h"

#define JOYSTICK_RESOLUTION 	100

static char InputFileName[512] 	= "test.cmu";
static unsigned int rows 		= 143;
static unsigned int cols 		= 176;
static int image_write_flag 	= 0;
static int image_update_count 	= 0;
static Image cmu_image;
static Image processed_image;

static int main_window			= 0;
static int menid				= 0;
static int submenid				= 0;
static int filterid				= 0;
static int loop 				= 0;
static int run 					= 0;
static int display_proc_image	= 0;

static int joy_x_val 			= 0;
static int joy_y_val 			= 0;

static char text1[100] 			= "x: 128";
static char text2[100] 			= "y: 128";
static char text3[100]			= "";

Serial serial;

int filterImage(int value)
{
	display_proc_image = 0;
	if (value == 2)	// sobel horizontal
	{
		hsobelImage(&cmu_image, &processed_image);
		display_proc_image = 1;
	}
	else if (value == 3)	// sobel vertical
	{
		vsobelImage(&cmu_image, &processed_image);
		display_proc_image = 1;
	}
	else if (value == 4)	// blur
	{
		blurImage(&cmu_image, &processed_image);
		display_proc_image = 1;
	}
	else if (value == 5)	// sharpen
	{
		sharpenImage(&cmu_image, &processed_image);
		display_proc_image = 1;
	}
	else if (value == 6)	// sinc upsample
	{
		sinc2XImage(&cmu_image, &processed_image);
		display_proc_image = 1;
 	}
	else					// no filter
	{
		display_proc_image = 0;
	}
	return display_proc_image;
}

void menu(int value)
{
	if (value == 0)
	{
		glutDestroyWindow(main_window);
		exit(0);
	}
	else if (value == 1)	// original image
	{
		filterid = 1;
		display_proc_image = filterImage(filterid);
	}
	else if (value == 2)	// sobel horizontal
	{
		filterid = 2;
		display_proc_image = filterImage(filterid);
	}
	else if (value == 3)	// sobel vertical
	{
		filterid = 3;
		display_proc_image = filterImage(filterid);
	}
	else if (value == 4)	// blur
	{
		filterid = 4;
		display_proc_image = filterImage(filterid);
	}
	else if (value == 5)	// sharpen
	{
		filterid = 5;
		display_proc_image = filterImage(filterid);
	}
	else if (value == 6)	// sinc upsample
	{
		filterid = 6;
		display_proc_image = filterImage(filterid);
 	}
	else if (value == 7)
	{
		loop = 1;
	}
	else if (value == 8)
	{
		cmu_image.WriteCMUfile(InputFileName);
	}
  
	glutPostRedisplay();
}

void createMenu(void)
{
	// Create a submenu, this has to be done first.
	submenid = glutCreateMenu(menu);
	glutAddMenuEntry("sobel horizontal" , 2);
	glutAddMenuEntry("sobel vertical"   , 3);
	glutAddMenuEntry("blur"             , 4);
	glutAddMenuEntry("sharpen"          , 5);
	glutAddMenuEntry("sinc upsample"    , 6);

	// Create the menu, this menu becomes the current menu
	menid = glutCreateMenu(menu);
	glutAddMenuEntry("Original Image", 1);
	glutAddSubMenu("Image Filters", submenid);
	glutAddMenuEntry("Loop", 7);
	glutAddMenuEntry("Save", 8);
	glutAddMenuEntry("Quit", 0);

	// Let the menu respond on the right mouse button
	glutAttachMenu(GLUT_RIGHT_BUTTON);
}

void initGUI(void)
{   
	createMenu();
	glClearColor (0.0, 0.0, 0.0, 0.0);
	glShadeModel(GL_FLAT);
	glRotatef(3.0, 1.0, 0.0, 0.0);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
}

void display(void)
{
	unsigned int i;
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	glRasterPos2i(0, 0);
	if (display_proc_image)
	{
		glutReshapeWindow(processed_image.width, processed_image.height);
		glDrawPixels(processed_image.width, processed_image.height, GL_RGB, GL_UNSIGNED_BYTE, processed_image.rgb_data);
	}
	else
	{
		glutReshapeWindow(cmu_image.width, cmu_image.height);
		glDrawPixels(cmu_image.width, cmu_image.height, GL_RGB, GL_UNSIGNED_BYTE, cmu_image.rgb_data);
	}
	glFlush();
	glPopMatrix();

	/*** Render the live character array 'text' ***/
	glColor3ub(255, 0, 0);
	glRasterPos2i(10, 40);
	for (i = 0; i < strlen(text1); i++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, text1[i]);
	glRasterPos2i(10, 20);
	for (i = 0; i < strlen(text2); i++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, text2[i]);
	glRasterPos2i(10, 60);
	for (i = 0; i < strlen(text3); i++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, text3[i]);

   glutSwapBuffers();
}

void reshape(int w, int h)
{
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0.0, (GLdouble) w, 0.0, (GLdouble) h);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

void processNormalKeys(unsigned char key, int x, int y) 
{
	switch (key)
	{
	case 'p':
		display();
		break;
	case 27: 
		exit(0);
	}
}

void Joystick(unsigned state,       /* I - Button state */
			  int      x,           /* I - X position (-1000 to 1000) */
			  int      y,           /* I - Y position (-1000 to 1000) */
			  int      z)           /* I - Z position (-1000 to 1000) */
{
	joy_x_val = x;
	joy_y_val = y;

	if (state & GLUT_JOYSTICK_BUTTON_A)
	{
		strcpy(text3, "Button 1");
		run = 1;
	}
	else if (state & GLUT_JOYSTICK_BUTTON_B)
	{
		strcpy(text3, "Button 2");
		loop = 0;
		run = 0;
	}
	else
		strcpy(text3, "");
}

void updateServo(int *servo_x_results, int *servo_y_results)
{
	static int servo_x_val = 128*JOYSTICK_RESOLUTION;
	static int servo_y_val = 128*JOYSTICK_RESOLUTION;

	// linear gain for joystick movement
	if ((joy_x_val > 0) && (servo_x_val < 255*JOYSTICK_RESOLUTION))
		servo_x_val++;
	if ((joy_x_val < 0) && (servo_x_val > 0))
		servo_x_val--;
	if ((joy_y_val > 0) && (servo_y_val < 255*JOYSTICK_RESOLUTION))
		servo_y_val++;
	if ((joy_y_val < 0) && (servo_y_val > 0))
		servo_y_val--;

	// accelerated gain for higher joystick value
	if ((joy_x_val > 500) && (servo_x_val < 255*JOYSTICK_RESOLUTION))
		servo_x_val += (joy_x_val*JOYSTICK_RESOLUTION)/1000;
	if ((joy_x_val < -500) && (servo_x_val > 0))
		servo_x_val += (joy_x_val*JOYSTICK_RESOLUTION-1)/1000;
	if ((joy_y_val > 500) && (servo_y_val < 255*JOYSTICK_RESOLUTION))
		servo_y_val += (joy_y_val*JOYSTICK_RESOLUTION)/1000;
	if ((joy_y_val < -500) && (servo_y_val > 0))
		servo_y_val += (joy_y_val*JOYSTICK_RESOLUTION-1)/1000;

	serial.sendServo(servo_x_val/JOYSTICK_RESOLUTION, SERVO_X_DIRECTION);
	serial.sendServo(servo_y_val/JOYSTICK_RESOLUTION, SERVO_Y_DIRECTION);
		
	*servo_x_results = servo_x_val/JOYSTICK_RESOLUTION;
	*servo_y_results = servo_y_val/JOYSTICK_RESOLUTION;
}

void idlefunc(void)
{
	int i, j, k;
	int servo_x, servo_y;
	char string_value[255];
	
	if (glutGetWindow() != main_window) 
		glutSetWindow(main_window);  

	if (run && (image_update_count >= 100))
	{
		if (!image_write_flag)
		{
			image_write_flag = 1;
			serial.grabImage2(InputFileName);
			image_write_flag = 0;
		}
		
		k = 7;
		for (j = 0; j < serial.height; j++) 
		{
			k++;
	        for (i = 0; i < serial.width; i++) 
			{
				cmu_image.rgb_data[(serial.width*2*(serial.height-j-1)+(i*2))*3+0] = serial.cmu_image2[k];	// red
				cmu_image.rgb_data[(serial.width*2*(serial.height-j-1)+(i*2))*3+3] = serial.cmu_image2[k];	// red
				cmu_image.rgb_data[(serial.width*2*(serial.height-j-1)+(i*2))*3+1] = serial.cmu_image2[k];	// green
				cmu_image.rgb_data[(serial.width*2*(serial.height-j-1)+(i*2))*3+4] = serial.cmu_image2[k];	// green
				cmu_image.rgb_data[(serial.width*2*(serial.height-j-1)+(i*2))*3+2] = serial.cmu_image2[k];	// blue
				cmu_image.rgb_data[(serial.width*2*(serial.height-j-1)+(i*2))*3+5] = serial.cmu_image2[k];	// blue
				cmu_image.Set(i*2, j, (int)serial.cmu_image2[k]);
				cmu_image.Set(i*2+1, j, (int)serial.cmu_image2[k]);
				k++;
	        }
	    }
		display_proc_image = filterImage(filterid);
		if (!loop) run = 0;
		image_update_count = 0;
	}
	image_update_count++;

	updateServo(&servo_x, &servo_y);
	sprintf(string_value,"x: %d", servo_x);
	strcpy(text1, string_value);
	sprintf(string_value,"y: %d", servo_y);
	strcpy(text2, string_value);
	glutPostRedisplay();
}

int main(int argc, char** argv)
{
	serial.Initialize();
//	cmu_image.width = cols;
//	cmu_image.height = rows;
//	cmu_image.Allocate(cols, rows);
	cmu_image.ReadCMUfile(InputFileName);
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(514, 514);
	glutInitWindowPosition(100, 100);
	main_window = glutCreateWindow("Robot GUI");
	initGUI();
	glutReshapeWindow(cols, rows);
	glutDisplayFunc(display);
	glutIdleFunc(idlefunc);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(processNormalKeys);
	glutJoystickFunc(Joystick, 1);
	glutMainLoop();
	return 0; 
}
